from flask import Flask, abort, request, jsonify
from tempfile import NamedTemporaryFile
from datetime import datetime

from dotenv import load_dotenv

import os
import sys
import requests
import base64
import io
import time
import glob
import traceback

from datetime import datetime

from wav_recorder import WavRecorder
from wav_composer import WavComposer
from channel_connector import ChannelConnector

load_dotenv()

CLIENT_URL = os.getenv('CLIENT_URL') or ''
HARPYIA_URL = os.getenv('HARPYIA_URL') or ''

app = Flask(__name__)

file_number = 1
temp_directory = 'temp/'
recordings_directory = 'recordings/'

def prepare_dirs():
    os.makedirs(os.path.dirname(temp_directory), exist_ok=True)
    os.makedirs(os.path.dirname(recordings_directory), exist_ok=True)
    files = glob.glob(temp_directory + "/*")
    for f in files:
            os.remove(f)

prepare_dirs()

wav_recorder = WavRecorder(recordings_directory)
wav_composer = WavComposer(16000)
channel_connector = ChannelConnector(16000)

@app.route("/")
def hello():
    return "To recognize an audio file, upload it using a POST request with '/save_audio' route."

@app.route("/time")
def get_time():
    current_timestamp = int(time.time())
    return {"current_timestamp": current_timestamp}

@app.route("/aid/ready", methods=['POST'])
def count():
    # requests.post(CLIENT_URL + '/state', json={'state': 1})
    return jsonify({}), 200

@app.route('/aid/transcript', methods=['POST'])
def handler():
    data = request.json
    audio_content = data['audio']['content']
    channel = data['audio']['channel']

    data['audio']['content'] = ''
    print(data, file=sys.stderr)

    initial_time = data['audio']['initialTime']
    micros = data['audio']['micros']

    # Decode base64 audio content
    audio_bytes = base64.b64decode(audio_content)

    # Save the audio as a WAV file
    buf = io.BytesIO()
    wav_composer.compose(buf, audio_bytes, channel=channel)
    buf.seek(0)
            
    try:
        requests.post(CLIENT_URL + '/state', json={'state': 0})

        files = {'file': buf}
        response = requests.post(HARPYIA_URL + '/recognize', files=files)
        if response.status_code == 200:
            data = response.json()
            results = data.get('results', [])
            transcript = ''

            for entity in results:
                transcript = f'{transcript} {entity.get("transcript")}'

            record_time = initial_time + int(micros / 1000000)
            record_time_str = datetime.fromtimestamp(record_time).strftime('%Y-%m-%d %H:%M:%S')
            record_millis_str = str(int(micros / 1000) % 1000).zfill(3)
            record_micros_str = str(micros % 1000).zfill(3)
            message = f'[{record_time_str}:{record_millis_str}:{record_micros_str}] {transcript}'
            print(message, file=sys.stderr)

            requests.post(CLIENT_URL + '/message', json={'message': message, 'results': entity.get('results')})

            return jsonify({'message': transcript})
        else:
            return jsonify({'error': 'Error occurred on Harpyia'}), 500
    except Exception as e:
        return jsonify({'error': str(e)}), 500

@app.route('/aid/save', methods=['POST'])
def save_audio():
    try:
        data = request.json
        audio_content = data['audio']['content']

        micros = data['audio']['micros']
        initial_time = data['audio']['initialTime']
        record_time = initial_time + int(micros / 1000000)
        record_millis_str = str(int(micros / 1000) % 1000).zfill(3)
        record_micros_str = str(micros % 1000).zfill(3)
        recording_time = datetime.fromtimestamp(record_time).strftime("%d_%m_%Y__%H_%M_%S")
        recording_time += f'_{record_millis_str}_{record_micros_str}'

        channel = data['audio']['channel']

        save_path = compose_save_path(recording_time, channel)
        audio_bytes = base64.b64decode(audio_content)

        wav_composer.compose(save_path, audio_bytes)
        audio_bytes = wav_composer.remove_initial_segment(audio_bytes, 100)
        channel_connector.add_audio(audio_bytes, channel)

        return jsonify({'status': 'success', 'message': 'Audio saved successfully'})
    except Exception as e:
        print(traceback.format_exc(), file=sys.stderr)
        return jsonify({'status': 'error', 'message': str(e)})

def compose_save_path(recording_time, channel):
    global file_number
    
    save_path = f'{temp_directory}saved_audio{str(file_number)}__channel{channel}__{recording_time}.wav'
    print(f'The audio file will be saved to {save_path}', file=sys.stderr)

    file_number += 1
    return save_path

@app.route("/recorder/client/start")
def start_recording():
    try:
        data = request.json
        audio_content = data['content']

        wav_recorder.start_recording()

        return jsonify({'status': 'success', 'message': 'Audio file saved'})
    except Exception as e:
        return jsonify({'status': 'error', 'message': str(e)})

@app.route("/recorder/cheeze/check")
def check_recording():
    try:
        return jsonify({'status': 'success', 'is_recording': wav_recorder.is_recording()})
    except Exception as e:
        return jsonify({'status': 'error', 'message': str(e)})

@app.route("/recorder/cheeze/send_header", methods=['POST'])
def receive_header():
    try:
        data = request.json
        audio_header = data['header']

        wav_recorder.set_header(audio_header)

        return jsonify({'status': 'success', 'message': 'Header received'})
    except Exception as e:
        return jsonify({'status': 'error', 'message': str(e)})

@app.route("/recorder/cheeze/send_part", methods=['POST'])
def receive_part():
    try:
        data = request.json
        audio_content = data['content']

        wav_recorder.append_data(audio_content)

        return jsonify({'status': 'success', 'message': 'Content part received'})
    except Exception as e:
        return jsonify({'status': 'error', 'message': str(e)})

@app.route("/recorder/client/stop")
def stop_recording():
    try:
        data = request.json
        audio_content = data['content']

        buf = io.BytesIO()
        wav_recorder.save_recording(buf=buf)
        buf.seek(0)

        return jsonify({'status': 'success', 'content': buf})
    except Exception as e:
        return jsonify({'status': 'error', 'message': str(e)})

